import os
import sys

from django.views.generic.edit import FormView
from django.shortcuts import render, redirect
from django.contrib.auth import get_user
from django.http import HttpResponse, Http404, JsonResponse
from django.db import transaction

from galeria.settings import GALLERY_INFO, LANGUAGE_CODE

from fotos.models import Album, Photo, UploadFile
from fotos.forms import FileUpload
from fotos.util import get_image_metadata, normalize

from datetime import datetime
from babel.dates import get_month_names

def get_month_name(ts, capitalize=False):
    locale = LANGUAGE_CODE.replace('-', '_')
    name = get_month_names(locale=locale)[ts.month]
    if capitalize:
        name = name.capitalize()
    return name

def _get_year_album(year):
    return Album.objects.filter(the_parent=None, slug=year).first()

def _get_month_album(ya, month):
    return Album.objects.filter(the_parent=ya, slug=month).first()

def _get_event_album(ma, event):
    return Album.objects.filter(the_parent=ma, slug=event).first()

def _get_photo(ea, photo):
    return Photo.objects.filter(the_album=ea, basename=photo).first()

def _get_years():
    years = Album.objects.filter(the_parent=None).order_by('first_timestamp')
    if len(years) == 0:
        return None
    return years

def _get_months(ya):
    return Album.objects.filter(the_parent=ya).order_by('first_timestamp')

def _get_events(ma):
    return Album.objects.filter(the_parent=ma).order_by('first_timestamp')

def _get_photos(ea):
    return Photo.objects.filter(the_album=ea).order_by('timestamp')

def _get_prev_year(ya):
    return None

def _get_next_year(ya):
    return None

def _get_prev_month(ma):
    return None

def _get_next_month(ma):
    return None

def _get_prev_event(ea):
    return None

def _get_next_event(ea):
    return None

def _get_prev_photo(ph):
    return None

def _get_next_photo(ph):
    return None

def main(request):
    page_title = 'Años'
    albums = _get_years()
    if albums is None:
        page_title = 'La galería está vacía'
    attrs = { 'albums': albums,
              'breadcrumbs': [],
              'gallery': GALLERY_INFO,
              'navigation': { 'prev': None, 'next': None, 'up': None },
              'now_year': datetime.now().year,
              'page_title': page_title,
              'prefix': '/galeria',
              'user': get_user(request) }
    return render(request, 'album.html', attrs)

def year(request, year):
    ya = _get_year_album(year)
    if ya is None:
        raise Http404(year + ' does not exist')
    attrs = { 'album': ya,
              'albums': _get_months(ya),
              'breadcrumbs': [ { 'link': ya.get_url_path(),
                                 'name': 'Año ' + year } ],
              'gallery': GALLERY_INFO,
              'navigation': { 'prev': _get_prev_year(ya),
                              'next': _get_next_year(ya),
                              'up': '/galeria' },
              'now_year': datetime.now().year,
              'page_title': 'Año ' + year,
              'prefix': '/galeria/' + year,
              'user': get_user(request) }
    return render(request, 'album.html', attrs)

def month(request, year, month):
    ya = _get_year_album(year)
    ma = _get_month_album(ya, month)
    if ya is None or ma is None:
        raise Http404(year + '/' + month + ' does not exist')
    attrs = { 'album': ma,
              'albums': _get_events(ma),
              'breadcrumbs': [ { 'link': ya.get_url_path(),
                                 'name': ya.name },
                               { 'link': ma.get_url_path(),
                                 'name': ma.name }  ],
              'gallery': GALLERY_INFO,
              'navigation': { 'prev': _get_prev_month(ma),
                              'next': _get_next_month(ma),
                              'up':  ya.get_url_path() },
              'now_year': datetime.now().year,
              'page_title': ('Mes de %s, %d' % \
                             ( get_month_name(ma.first_timestamp),
                               ya.first_timestamp.year )),
              'prefix': '/galeria/' + year + '/' + month,
              'user': get_user(request) }
    return render(request, 'album.html', attrs)

def event(request, year, month, event):
    ya = _get_year_album(year)
    ma = _get_month_album(ya, month)
    ea = _get_event_album(ma, event)
    if ya is None or ma is None or ea is None:
        raise Http404(year + '/' + month + '/' + event + ' does not exist')
    attrs = { 'album': ea,
              'breadcrumbs': [ { 'link': ya.get_url_path(),
                                 'name': ya.name },
                               { 'link': ma.get_url_path(),
                                 'name': ma.name },
                               { 'link': ea.get_url_path(),
                                 'name': ea.name } ],
              'gallery': GALLERY_INFO,
              'navigation': { 'prev': _get_prev_event(ea),
                              'next': _get_next_event(ea),
                              'up':  ma.get_url_path() },
              'now_year': datetime.now().year,
              'page_title': ea.name,
              'photos': _get_photos(ea),
              'prefix': '/galeria/' + year + '/' + month + '/' + event,
              'user': get_user(request) }
    return render(request, 'album.html', attrs)

def photo(request, year, month, event, photo):
    ya = _get_year_album(year)
    ma = _get_month_album(ya, month)
    ea = _get_event_album(ma, event)
    ph = _get_photo(ea, photo)
    if ya is None or ma is None or ea is None or ph is None:
        raise Http404(year + '/' + month + '/' +
                      event + '/' + photo + ' does not exist')
    attrs = { 'breadcrumbs': [ { 'link': ya.get_url_path(),
                                 'name': ya.name },
                               { 'link': ma.get_url_path(),
                                 'name': ma.name },
                               { 'link': ea.get_url_path(),
                                 'name': ea.name },
                               { 'link': ph.get_url_path(),
                                 'name': ph.title } ],
              'gallery': GALLERY_INFO,
              'navigation': { 'prev': _get_prev_photo(ph),
                              'next': _get_next_photo(ph),
                              'up':  ea.get_url_path() },
              'now_year': datetime.now().year,
              'page_title': ph.title,
              'photo': ph,
              'prefix': '/galeria/' + year + '/' + month + '/' + event,
              'user': get_user(request) }
    return render(request, 'photo.html', attrs)

@transaction.atomic
def _get_photo_album(metadata):
    ts = metadata['timestamp']
    sy = '%04d' % ts.year
    sm = '%02d' % ts.month
    year_album = Album.objects.filter(the_parent=None, slug=sy).first()
    if year_album is None:
        year_album = Album(the_parent = None,
                           name = 'Año ' + sy,
                           slug = sy,
                           first_timestamp = ts,
                           last_timestamp = ts,
                           time_offset = metadata['time_offset'],
                           the_photo = None)
        year_album.save()
    month_album = Album.objects.filter(the_parent=year_album, slug=sm).first()
    if month_album is None:
        month_album = Album(the_parent = year_album,
                            name = get_month_name(ts, capitalize=True),
                            slug = sm,
                            first_timestamp = ts,
                            last_timestamp = ts,
                            time_offset = metadata['time_offset'],
                            the_photo = None)
        month_album.save()
    pa_slug = normalize(metadata['album'])
    event_album = Album.objects.filter(the_parent=month_album,
                                       slug=pa_slug).first()
    if event_album is None:
        event_album = Album(the_parent = month_album,
                            name = metadata['album'],
                            slug = pa_slug,
                            first_timestamp = ts,
                            last_timestamp = ts,
                            time_offset = metadata['time_offset'],
                            the_photo = None)
        event_album.save()
    return year_album, month_album, event_album

@transaction.atomic
def _add_photo(upload_file):
    path = '/' + upload_file.files.name
    upload_file.delete()
    md = get_image_metadata(path)
    if md is None:
        os.remove(path)
        return
    year_album, month_album, event_album = _get_photo_album(md)
    photo = Photo(title = md['title'],
                  description = md['description'],
                  the_album = event_album,
                  location = md['location'],
                  latitude = md['latitude'],
                  longitude = md['longitude'],
                  width = md['width'],
                  height = md['height'],
                  view_width = md['view_width'],
                  view_height = md['view_height'],
                  thumb_width = md['thumb_width'],
                  thumb_height = md['thumb_height'],
                  timestamp = md['timestamp'],
                  time_offset = md['time_offset'])
    photo.fill_aux_data()
    photo.save()
    if event_album.first_timestamp > photo.timestamp:
        event_album.first_timestamp = photo.timestamp
    if event_album.last_timestamp < photo.timestamp:
        event_album.last_timestamp = photo.timestamp
    if event_album.the_photo is None:
        event_album.the_photo = photo
    event_album.save()
    if month_album.the_photo is None:
        month_album.the_photo = photo
        month_album.save()
    if year_album.the_photo is None:
        year_album.the_photo = photo
        year_album.save()
    os.remove(path)

def admin_upload(request):
    if request.method == 'POST':
        form = FileUpload(request.POST, request.FILES)
        files = request.FILES.getlist('files')
        if form.is_valid():
            for f in files:
                upload_file = UploadFile(files=f)
                upload_file.save()
                _add_photo(upload_file)
            return JsonResponse({'data': 'Data uploaded'})
        else:
            return JsonResponse({'data': 'Something went wrong'})
    else:
        form = FileUpload()
        return render(request, 'upload.html',
                      { 'form': form,
                        'breadcrumbs': [],
                        'gallery': GALLERY_INFO,
                        'navigation': { 'prev': None,
                                        'next': None,
                                        'up': None },
                        'now_year': datetime.now().year,
                        'page_title': 'Subir fotos' })
